<?php

declare(strict_types=1);

namespace App\Controller\Admin\Ajax;

use App\Entity\DeviceOsType;
use App\Entity\DeviceOsVersion;
use App\Form\FwType;
use App\Repository\DeviceOsVersionRepository;
use Doctrine\Persistence\ManagerRegistry;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController as Controller;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;

/**
 * @Route("/admin/ajax/fw", name="ajax_fw_")
 */
class FwController extends Controller
{

    /**
     * Create new Firmware
     * @Route("admin/firmware/create", methods={"POST"}, name="create")
     * @param Request $request
     * @param ManagerRegistry $managerRegistry
     * @return Response
     */
    public function createFirmwareAction(Request $request, ManagerRegistry $managerRegistry): Response {

        $deviceOsTypes = $managerRegistry->getRepository(DeviceOsType::class)->findAll();

        $osVersion = new DeviceOsVersion();
        $form = $this->createForm(FwType::class, ['osTypes' => $deviceOsTypes]);
        $form->handleRequest($request);
        $errors = array();

        if ($form->isSubmitted()) {
            $errorsValidator = [];
            foreach ($errorsValidator as $error) {
                array_push($errors, $error->getMessage());
            }

            if (count($errors) == 0) {

                $formData = $form->getData();

                $osType = $managerRegistry->getRepository(DeviceOsType::class)->findOneBy(['id' => $formData['os_type']]);

                $em = $managerRegistry->getManager();
                $osVersion->setDescription($formData['version']);
                $osVersion->setOsType($osType);
                $em->persist($osVersion);
                $em->flush();

                return new JsonResponse(array(
                    'code' => 200,
                    'message' => 'success',
                    'errors' => array('errors' => array(''))),
                    200);
            }

        }

        return new JsonResponse(array(
            'code' => 400,
            'message' => 'error',
            'errors' => array('errors' => $errors)),
            400);
    }

    /**
     * Get All Fw versions
     * @Route("/all", methods={"GET"}, name="get_all")
     * @param Request $request
     * @param DeviceOsVersionRepository $deviceOsVersionRepository
     * @return Response
     */
    public function getAllLabelsAction(Request $request, DeviceOsVersionRepository $deviceOsVersionRepository): Response {

        if ($request->isXmlHttpRequest()) {

            $fws = $deviceOsVersionRepository->getAvailableAndApprovedVersions();
            $result = array();

            foreach ($fws as $fw) {
                $result[$fw->getId()] = ['value' => $fw->getVersion()];
            }

            return new JsonResponse($result);

        }

        return new Response('Bad request.', 400);

    }

}