<?php

declare(strict_types=1);

namespace App\Entity;

use App\Entity\AgentProfile\AgentProfile;
use App\Entity\Device\DeviceNote;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;

/**
 * @ORM\Entity(repositoryClass=\App\Repository\RealDeviceRepository::class)
 * @ORM\Table(name="devices")
 *
 */
class RealDevice
{

    const ENTITY_TABLE_NAME = 'devices';

    /**
     * @var int
     * @ORM\Column(name="id_device", type="integer")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private int $id;

    /**
     * @var string
     * @ORM\Column(name="devid", type="string", length=8, nullable=false, unique=true)
     */
    public string $devId;

    /**
     * @var string|null
     * @ORM\Column(type="string", length=32, nullable=true)
     */
    public ?string $name;

    /**
     * @var DeviceType
     * @ORM\ManyToOne(targetEntity="DeviceType", inversedBy="devices")
     * @ORM\JoinColumn(name="dev_type")
     */
    public DeviceType $devType;

    /**
     * @var DeviceOsVersion
     * @ORM\ManyToOne(targetEntity="DeviceOsVersion", inversedBy="devices")
     * @ORM\JoinColumn(name="os_ver")
     */
    public ?DeviceOsVersion $osVer = null;

    /**
     * @var DeviceOsVersion|null
     * @ORM\ManyToOne(targetEntity="DeviceOsVersion", inversedBy="devicesDesired")
     * @ORM\JoinColumn(name="os_ver_desired")
     */
    public ?DeviceOsVersion $osVerDesired = null;

    /**
     * @var string|null
     * @ORM\Column(name="macaddr", type="string", length=12, nullable=true)
     */
    public ?string $macAddr;

    /**
     * @var string|null
     * @ORM\Column(name="serno", type="string", length=32, nullable=true)
     */
    public ?string $serial;

    /**
     * @var string|null
     * @ORM\Column(name="iccid", type="string", length=32, nullable=true)
     */
    public ?string $iccid;

    /**
     * @var string|null
     * @ORM\Column(name="active_sim", type="string", length=32, nullable=true)
     */
    public ?string $activeSim;

    /**
     * @var string|null
     * @ORM\Column(type="string", length=64, nullable=true)
     */
    public ?string $hostname;

    /**
     * @var string|null
     * @ORM\Column(type="string", length=64, nullable=true)
     */
    public ?string $hostnameDesired;

    /**
     * @var string|null
     * @ORM\Column(type="string", length=16, nullable=true)
     */
    public ?string $uuid;

    /**
     * @var DeviceStats
     * @ORM\OneToOne(targetEntity="App\Entity\DeviceStats")
     * @ORM\JoinColumn(name="id_device", referencedColumnName="id_device")
     */
    public DeviceStats $stats;

    /**
     * @var bool
     * @ORM\Column(type="boolean", nullable=false, options={"default": false})
     */
    public bool $deleted;

    /**
     * @var int
     * @ORM\Column(type="integer", nullable=false)
     */
    public int $guiVersion;

    /**
     * @var AgentProfile
     * @ORM\ManyToOne(targetEntity="App\Entity\AgentProfile\AgentProfile", inversedBy="devices")
     * @ORM\JoinColumn(name="agent_profile")
     */
    public AgentProfile $agentProfile;

    /**
     * @var Collection
     * @ORM\OneToMany (targetEntity="App\Entity\MonitoringGroupMapping", mappedBy="device", cascade={"persist"})
     * @ORM\JoinColumn(name="id")
     */
    public Collection $monitoringGroupMappings;

    /**
     * @var Collection
     * @ORM\OneToMany (targetEntity="App\Entity\DeviceNetDev", mappedBy="device", cascade={"persist"})
     * @ORM\JoinColumn(name="id_device")
     */
    public Collection $netDevices;

    /**
     * @var DeviceNote|null
     * @ORM\OneToOne (targetEntity="App\Entity\Device\DeviceNote", mappedBy="device", cascade={"persist"})
     * @ORM\JoinColumn(name="id_device", referencedColumnName="id_device", nullable=true)
     */
    public ?DeviceNote $noteDevice;

    /**
     * @var Collection
     * @ORM\OneToMany (targetEntity="App\Entity\Alert\Alert", mappedBy="targetDevice", cascade={"persist"})
     * @ORM\JoinColumn(name="id_device")
     */
    public Collection $alerts;

    /**
     * @var Collection
     * @ORM\OneToMany (targetEntity="LabelDeviceMap", mappedBy="device", cascade={"persist"})
     * @ORM\JoinColumn(name="id_device")
     */
    public Collection $labels;

    /**
     * @var ConfigProfile|null
     * @ORM\ManyToOne(targetEntity="App\Entity\ConfigProfile", inversedBy="devices", cascade={"persist"})
     * @ORM\JoinColumn(name="config_profile", nullable=true)
     */
    public ?ConfigProfile $configProfile = null;

    /**
     * @var ConfigProfile|null
     * @ORM\OneToOne (targetEntity="App\Entity\ConfigProfile", mappedBy="automatic", cascade={"persist"})
     * @ORM\JoinColumn(name="automatic_profile", nullable=true)
     */
    public ?ConfigProfile $configProfileAutomatic;

    /**
     * @var Collection
     * @ORM\OneToMany (targetEntity="App\Entity\TunnelWeb", mappedBy="device", cascade={"persist"})
     * @ORM\JoinColumn(name="id_device")
     */
    public Collection $webTunnels;

    /**
     * @var Collection
     * @ORM\OneToMany (targetEntity="App\Entity\DataChangeLog", mappedBy="device", cascade={"persist"})
     * @ORM\JoinColumn(name="id_device")
     */
    public Collection $dataChangeLogs;

    /**
     * @var Collection
     * @ORM\OneToMany (targetEntity="App\Entity\DeviceUserModule", mappedBy="device", cascade={"persist"})
     * @ORM\JoinColumn(name="id_device")
     */
    public Collection $userModules;

    /**
     * Device constructor
     */
    public function __construct()
    {
        $this->monitoringGroupMappings = new ArrayCollection();
        $this->netDevices = new ArrayCollection();
        $this->alerts = new ArrayCollection();
        $this->labels = new ArrayCollection();
        $this->webTunnels = new ArrayCollection();
        $this->dataChangeLogs = new ArrayCollection();
        $this->userModules = new ArrayCollection();
    }

    /**
     * @return int|null
     */
    public function getId(): ?int {

        return $this->id;

    }

    /**
     * @return string|null
     */
    public function getDevId(): ?string {

        return $this->devId;

    }

    /**
     * @return string
     */
    public function getName(): ?string {

        return $this->name;

    }

    /**
     * @return DeviceType
     */
    public function getDevType(): DeviceType {

        return $this->devType;

    }

    /**
     * @return DeviceOsVersion
     */
    public function getOsVer(): ?DeviceOsVersion {

        return $this->osVer;

    }

    /**
     * @return DeviceOsVersion
     */
    public function getOsVerDesired(): ?DeviceOsVersion {

        return $this->osVerDesired;

    }

    /**
     * @param DeviceOsVersion|null $osVersion
     */
    public function setOsVerDesired(?DeviceOsVersion $osVersion): void {

        $this->osVerDesired = $osVersion;

    }

    /**
     * @return string
     */
    public function getMacAddr(): ?string {

        return $this->macAddr;

    }

    /**
     * @return string
     */
    public function getSerial(): ?string {

        return $this->serial;

    }

    /**
     * @return string
     */
    public function getIccid(): ?string {

        return $this->iccid;

    }

    /**
     * @return string
     */
    public function getHostname(): ?string {

        return $this->hostname;

    }

    /**
     * @return string
     */
    public function getHostnameDesired(): ?string {

        return $this->hostnameDesired;

    }

    /**
     * @param string|null $hostname
     */
    public function setHostnameDesired(?string $hostname): void {

        $this->hostnameDesired = $hostname;

    }

    /**
     * @return string
     */
    public function getUuid(): ?string {

        return $this->uuid;

    }

    /**
     * @return DeviceStats
     */
    public function getStats(): ?DeviceStats {

        return $this->stats;

    }

    /**
     * @return mixed
     */
    public function getDeleted(): bool{

        return $this->deleted;

    }

    /**
     * @return int
     */
    public function getGuiVersion(): int {

        return $this->guiVersion;

    }

    /**
     * @param mixed $deleted
     */
    public function setDeleted(bool $deleted): void {

        $this->deleted = $deleted;

    }

    /**
     * @param int $version
     */
    public function setGuiVersion(int $version): void {

        $this->guiVersion = $version;

    }

    /**
     * @return ConfigProfile|null
     */
    public function getConfigProfile(): ?ConfigProfile {

        return $this->configProfile;

    }

    /**
     * @return ConfigProfile|null
     */
    public function getAgentProfile(): ?AgentProfile {

        return $this->agentProfile;

    }

    /**
     * @param AgentProfile $agentProfile
     * @return void
     */
    public function setAgentProfile(AgentProfile $agentProfile): void {

        $this->agentProfile = $agentProfile;

    }

    /**
     * @param ConfigProfile|null $configProfile
     */
    public function setConfigProfile(?ConfigProfile $configProfile): void {

        $this->configProfile = $configProfile;

    }

    /**
     * @return ConfigProfile
     */
    public function getConfigProfileAutomatic(): ?ConfigProfile {

        return $this->configProfileAutomatic;

    }

    /**
     * @return ArrayCollection
     */
    public function getNetDevices(): ArrayCollection {

        return $this->netDevices;

    }

    public function getNoteDevices(): ArrayCollection {

        return $this->noteDevices;

    }

    /**
     * @return Collection
     */
    public function getUserModules(): Collection {

        return $this->userModules;

    }

    /**
     * @return bool
     */
    public function isDeleted(): bool {

        return $this->deleted;

    }


}